#!/usr/bin/perl
#
################################################################################
#
# IPCop DigiTemp AddOn Web-Iface
#
# Copyright (C) 2006-2011 Olaf (weizen_42) Westrik
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA 
#
#
# Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der GNU General Public License, wie von der Free Software Foundation verffentlicht, weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder (nach Ihrer Option) jeder spteren Version. 
#
# Die Verffentlichung dieses Programms erfolgt in der Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 
#
# Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm erhalten haben. Falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
#
################################################################################
#
# For support post / read in http://www.ipcop-forum.de
#
# $Id: digitemp.cgi 529 2014-04-09 07:48:07Z weizen_42 $
#
# Revision history:
#
# 2006-09-xx created by weizen_42 at ipcop-forum dot de
# 2006-12-18  v0.9.2    check for updates, (re-)scan for sensors
# 2007-03-02  v0.9.3    Upgraded digitemp to v3.5.0
#                       minor modifications to Webgui
#                       sensor scanning uses -w option
# 2007-05-14  v0.9.4    Russian translation
# 2007-05-25  v0.9.5    revert -w
# 2007-05-28  v1.0.0    Minor modifications to Webgui
#

# Add entry in menu
# MENUENTRY status 059 "DigiTemp" "DigiTemp"

use strict;

# enable only the following on debugging purpose
#use warnings;
#use CGI::Carp 'fatalsToBrowser';

use LWP::UserAgent;

require '/usr/lib/ipcop/general-functions.pl';
require '/usr/lib/ipcop/lang.pl';
require '/usr/lib/ipcop/header.pl';

my $version = 'v2.0.3';

# checking for new version
my $addonname = 'digitemp';
my $onlineversion = '';
my $onlinelink = '';
my $timestamplastcheck = '/var/ipcop/addons/${addonname}/lastcheck';
my $flagdonotcheck = '/var/ipcop/addons/${addonname}/noversioncheck';


# path for graphs
my $path_graphs = "/home/httpd/html/graphs";
# path for values
my $path_values = "/var/log/digitemp.log";
# settings
my %digitemp_settings = ();
$digitemp_settings{'SENSORS'} = 0;
$digitemp_settings{'SHOWGRAPH'} = '';
$digitemp_settings{'GRAPHPERSENSOR'} = '';

&General::readhash('/var/ipcop/addons/digitemp/settings', \%digitemp_settings) if (-e '/var/ipcop/addons/digitemp/settings');

my %adapter = ();
my %showgraph = ();
my %values = ();
my $humidity = 0;
&ReadValues;

my $i = 0;
my $j = 0;
my $line = '';
my $key = '';

# get rid of used only once warnings
my @onlyonce = ( $Header::table1colour, $Header::table2colour );
undef @onlyonce;

&Header::showhttpheaders();
my %cgiparams=();
my @cgigraphs=();
$cgiparams{'ACTION'} = '';

&General::getcgihash(\%cgiparams);

$ENV{'QUERY_STRING'} =~ s/&//g;
@cgigraphs = split(/graph=/,$ENV{'QUERY_STRING'});
$cgigraphs[1] = '' unless defined $cgigraphs[1];


##########
#
# scan for sensors
#
##########
if ( $cgiparams{'ACTION'} eq $Lang::tr{'digitemp scan'} ) {
    system("/bin/touch /var/ipcop/addons/digitemp/rescan");
}

##########
#
# modify settings
#
##########
if ( $cgiparams{'ACTION'} eq $Lang::tr{'save'} ) {
    $digitemp_settings{'ADAPTER'} = $cgiparams{'ADAPTER'};
    $digitemp_settings{'SHOWGRAPH'} = $cgiparams{'SHOWGRAPH'};
    $digitemp_settings{'GRAPHPERSENSOR'} = $cgiparams{'GRAPHPERSENSOR'};

    foreach my $line (sort keys %cgiparams) {
        if ( index($line, "LABEL-") != -1 ) {
            $digitemp_settings{$line} = $cgiparams{$line};
        }
    }

    &General::writehash('/var/ipcop/addons/digitemp/settings', \%digitemp_settings);
}

$adapter{'DS9097'} = '';
$adapter{'DS9097U'} = '';
$adapter{'DS2490'} = '';
if ( $digitemp_settings{'ADAPTER'} ne '' ) {
    $adapter{$digitemp_settings{'ADAPTER'}} = "selected='selected'";
}

$showgraph{'day'} = '';
$showgraph{'week'} = '';
$showgraph{'month'} = '';
$showgraph{'year'} = '';
if ( $digitemp_settings{'SHOWGRAPH'} ne '' ) {
    $showgraph{$digitemp_settings{'SHOWGRAPH'}} = "selected='selected'";
}
else {
    $digitemp_settings{'SHOWGRAPH'} = 'day';
}

my $graphpersensor = '';
$graphpersensor = "checked='checked'" if ( $digitemp_settings{'GRAPHPERSENSOR'} eq 'on' );

&Header::openpage("DigiTemp", 1, '');
&Header::openbigbox('100%', 'left');

#
# Check for new version
#
&checkfornewversion($addonname, $version);
if ( $onlineversion ne '' ) {
    &Header::openbox('100%', 'left', $Lang::tr{'info'}, 'warning');
    print <<END
<table width="100%"><tr>
<td>$Lang::tr{'digitemp newversion'} <a href="$onlinelink" target="_blank"><b>$onlineversion</b></a></td>
</tr></table>
END
    ;
    &Header::closebox();
}


##########
#
# Box(es) with graphs
#
##########
if ( $cgigraphs[1] =~ /digitemp/ ) {
    &Header::openbox('100%', 'center', "$Lang::tr{'graph'}");
    if (-e "$path_graphs/digitemp-day.png") {
        print "<div align='left'><a href='/cgi-bin/digitemp.cgi'><img src='/images/back.png' alt='$Lang::tr{'back'}' title='$Lang::tr{'back'}' /></a></div><hr />\n";
        print "<img src='/graphs/digitemp-day.png' border='0' alt='' /><hr />";
        print "<img src='/graphs/digitemp-week.png' border='0' alt='' /><hr />";
        print "<img src='/graphs/digitemp-month.png' border='0' alt='' /><hr />";
        print "<img src='/graphs/digitemp-year.png' border='0' alt='' />";

        if ($humidity != 0) {
            print "<hr />";
            print "<img src='/graphs/digitemp-hum-day.png' border='0' alt='' /><hr />";
            print "<img src='/graphs/digitemp-hum-week.png' border='0' alt='' /><hr />";
            print "<img src='/graphs/digitemp-hum-month.png' border='0' alt='' /><hr />";
            print "<img src='/graphs/digitemp-hum-year.png' border='0' alt='' />";
        }
    }
    else {
        print $Lang::tr{'no information available'};
    }
    print "<hr /><div align='left'><a href='/cgi-bin/digitemp.cgi'><img src='/images/back.png' alt='$Lang::tr{'back'}' title='$Lang::tr{'back'}' /></a></div>\n";
    &Header::closebox(); 
}
elsif ($cgigraphs[1] ne '') {
    my $SNR = $cgigraphs[1];
    my $label = &Header::cleanhtml($digitemp_settings{"LABEL-$SNR"});

    &Header::openbox('100%', 'center', "$Lang::tr{'graph'} $label");
    if (-e "$path_graphs/digitemp-$SNR-day.png") {
        print "<div align='left'><a href='/cgi-bin/digitemp.cgi'><img src='/images/back.png' alt='$Lang::tr{'back'}' title='$Lang::tr{'back'}' /></a></div><hr />\n";
        print "<img src='/graphs/digitemp-$SNR-day.png' border='0' alt='' /><hr />";
        print "<img src='/graphs/digitemp-$SNR-week.png' border='0' alt='' /><hr />";
        print "<img src='/graphs/digitemp-$SNR-month.png' border='0' alt='' /><hr />";
        print "<img src='/graphs/digitemp-$SNR-year.png' border='0' alt='' />";

        if ( $values{$SNR}{'H'} != 0 ) {
            print "<hr />";
            print "<img src='/graphs/digitemp-$SNR-hum-day.png' border='0' alt='' /><hr />";
            print "<img src='/graphs/digitemp-$SNR-hum-week.png' border='0' alt='' /><hr />";
            print "<img src='/graphs/digitemp-$SNR-hum-month.png' border='0' alt='' /><hr />";
            print "<img src='/graphs/digitemp-$SNR-hum-year.png' border='0' alt='' />";
        }
    }
    else {
        print $Lang::tr{'no information available'};
    }
    print "<hr /><div align='left'><a href='/cgi-bin/digitemp.cgi'><img src='/images/back.png' alt='$Lang::tr{'back'}' title='$Lang::tr{'back'}' /></a></div>\n";
    &Header::closebox(); 
}
elsif ( $digitemp_settings{'GRAPHPERSENSOR'} eq 'on' ) {
    for($i = 0; $i < $digitemp_settings{'SENSORS'}; $i++) {
        my $SNR = $digitemp_settings{"SENSOR$i"};
        my $label = &Header::cleanhtml($digitemp_settings{"LABEL-$SNR"});

        &Header::openbox('100%', 'center', "$Lang::tr{'graph'} $label");
        if (-e "$path_graphs/digitemp-$SNR-$digitemp_settings{'SHOWGRAPH'}.png") {
            print "<a href='/cgi-bin/digitemp.cgi?graph=$SNR'>";
            print "<img src='/graphs/digitemp-$SNR-$digitemp_settings{'SHOWGRAPH'}.png' border='0' alt='' />";
            print "</a>";

            if ( $values{$SNR}{'H'} != 0 ) {
                print "<a href='/cgi-bin/digitemp.cgi?graph=$SNR'>";
                print "<img src='/graphs/digitemp-$SNR-hum-$digitemp_settings{'SHOWGRAPH'}.png?graph=digitemp' border='0' alt='' />";
                print "</a>";
            }
        }
        else {
            print $Lang::tr{'no information available'};
        }
        print "<br />\n";
        &Header::closebox(); 
    }
}
else {
    &Header::openbox('100%', 'center', "$Lang::tr{'graph'}");
    if (-e "$path_graphs/digitemp-day.png") {
        print "<a href='/cgi-bin/digitemp.cgi?graph=digitemp'>";
        print "<img src='/graphs/digitemp-$digitemp_settings{'SHOWGRAPH'}.png' border='0' alt='' />";
        print "</a>";

        if ($humidity != 0) {
            print "<br /><a href='/cgi-bin/digitemp.cgi?graph=digitemp'>";
            print "<img src='/graphs/digitemp-hum-$digitemp_settings{'SHOWGRAPH'}.png?graph=digitemp' border='0' alt='' />";
            print "</a>";
        }
    }
    else {
        print $Lang::tr{'no information available'};
    }
    print "<br />\n";
    &Header::closebox(); 
}


##########
#
# Box with config / settings
#
##########
if ( $cgigraphs[1] eq '' ) {
    &Header::openbox('100%', 'left', "$Lang::tr{'settings'}");
    print <<END
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
<table width='100%' border='0' cellspacing='1' cellpadding='0'>
<tr>
  <td align='left' width='25%'>$Lang::tr{'digitemp adapter'}:</td>
  <td align='left' width='25%'><select name='ADAPTER'>
END
    ;

    foreach $key ( sort(keys %adapter) ) {
        print "<option value='$key' ".$adapter{"$key"}.">$key</option>";
    }

    print <<END
    </select>
  </td>
  <td>&nbsp;</td>
  <td class='button1button'><input type='submit' name='ACTION' value='$Lang::tr{'digitemp scan'}' /></td>
  <td class='onlinehelp'>&nbsp;</td>
</tr>
<tr>
  <td align='left' width='25%'>$Lang::tr{'digitemp graph1'}:</td>
  <td align='left' width='25%'><input type='checkbox' name='GRAPHPERSENSOR' $graphpersensor/></td>
  <td colspan='2'>&nbsp;</td>
</tr>
<tr>
  <td align='left' width='25%'>$Lang::tr{'digitemp showgraph'}:</td>
  <td align='left' width='25%'><select name='SHOWGRAPH'>
END
    ;

    foreach $key ( sort(keys %showgraph) ) {
        print "<option value='$key' ".$showgraph{"$key"}.">".$Lang::tr{$key}."</option>";
    }

    print <<END
    </select>
  </td>
  <td colspan='2'>&nbsp;</td>
</tr>
</table>
<br /><hr /><br />
END
    ;

    print <<END
<table width='100%'>
<tr><td align='center' width='10%'><b>Sensor</b></td>
<td align='center' width='10%'><b>Temp (&#x00BA;C)</b></td>
<td align='center' width='10%'><b>Hum (%)</b></td>
<td align='left'><b>Label</b></td></tr>
END
    ;

    $i = 0;
    for($i = 0; $i < $digitemp_settings{'SENSORS'}; $i++) {
        print("<tr class='table".int(($i % 2) + 1)."colour'>");

        my $SNR = $digitemp_settings{"SENSOR$i"};
        my $label = &Header::cleanhtml($digitemp_settings{"LABEL-$SNR"});
        $j = $i + 1;
        print("<td align='center'>$j</td>");
        print("<td align='center'>$values{$i}{'T'}</td><td align='center'>$values{$i}{'H'}</td>\n");
        print("<td>&nbsp;<input type='text' name='LABEL-$SNR' value='$label' size='40' /></td></tr>\n");
    }
    print <<END
</table>

<table width='100%'>
<tr><td class='comment1button'>&nbsp;</td><td class='button1button'><input type='submit' name='ACTION' value='$Lang::tr{'save'}' /></td><td class='onlinehelp'>&nbsp;</td></tr>
</table></form>
END
    ;

    print <<END
<hr />
<table width='100%'>
<tr><td>&nbsp;</td><td align='right'><b><small><a href="http://www.digitemp.com" target="_blank">DigiTemp v3.6.0</a></small></b></td></tr>
<tr><td>&nbsp;</td><td align='right'><b><small><a href="http://www.ban-solms.de/t/IPCop.html" target="_blank">DigiTemp Addon $version</a></small></b></td></tr>
</table>
END
    ;

    &Header::closebox(); 
}


&Header::closebigbox();
&Header::closepage();


#
# load the sensor values
#
sub ReadValues
{
    # datafileformat:
    #   NR,SNR,Temp,Hum
    # 
    my @tmpfile = ();
    if ( open(FILE, "$path_values") ) {
        @tmpfile = <FILE>;
        close (FILE);
    }

    # populate devices list
    foreach $line ( @tmpfile ) {
        chomp($line);               # remove newline
        my @temp = split(/\,/,$line,4);
        next if ( $temp[0] eq '' );

        $values{$temp[0]}{'SNR'} = $temp[1];
        $values{$temp[0]}{'T'}   = $temp[2];
        $values{$temp[0]}{'H'}   = $temp[3];

        if ( $values{$temp[0]}{'H'} != -1 ) {
            $humidity = 1;
            $values{$temp[1]}{'H'} = $values{$temp[0]}{'H'};
        }
    }
}

sub checkfornewversion
{
  my $addon = shift;
  my $version = shift;
  $onlineversion = '';

  if ( -e $flagdonotcheck )
  {
    return;
  }

  # only check if we are online and last check was some time ago
  if ( (! -e '/var/ipcop/red/active') || (-e $timestamplastcheck) && (int(-M $timestamplastcheck) < 5) )
  {
    return;
  }

#workaround to suppress a warning when a variable is used only once
  my @dummy = ( $General::version );
  undef (@dummy);

  my $ua = LWP::UserAgent->new;
  $ua->timeout(120);
  $ua->agent("Mozilla/4.0 (compatible; IPCop $General::version; $version)");
  my $content = $ua->get("http://www.ban-solms.de/version/$addon");

  if ( $content->is_success )
  {
    # compare the versions, format is v1.2.3
    $content->content =~ /v(\d+).(\d+).(\d+)/;
    my $ver1 = $1;
    my $ver2 = $2;
    my $ver3 = $3;

    $version =~ /v(\d+).(\d+).(\d+)/;

    if ( ($ver1 > $1) || (($ver1 == $1) && ($ver2 > $2)) || (($ver1 == $1) && ($ver2 == $2) && ($ver3 > $3)) )
    {
      $onlineversion = "v$ver1.$ver2.$ver3";

      $content->content =~ /http(.*)/;
      $onlinelink = "http$1";
    }
    else
    {
      # no news, recheck in a couple of days
      system("touch $timestamplastcheck");
    }
  }
}
