#!/usr/bin/perl
#
################################################################################
#
# script to get readings from digitemp and dump into rrd
#
# Copyright (C) 2006-2007 Olaf (weizen_42) Westrik
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA 
#
#
# Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der GNU General Public License, wie von der Free Software Foundation verffentlicht, weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder (nach Ihrer Option) jeder spteren Version. 
#
# Die Verffentlichung dieses Programms erfolgt in der Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 
#
# Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm erhalten haben. Falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
#
################################################################################
#
# For support post / read in http://www.ipcop-forum.de
#
# $Id: digitemp.pl 462 2011-04-22 14:28:31Z weizen_42 $
#
# Revision history:
#
# 2006-09-09 created by weizen_42 at ipcop-forum dot de
# 2007-10-14 weizen_42        additional language hack(s)
#

use strict;
use RRDs;
require '/usr/lib/ipcop/general-functions.pl';
require '/usr/lib/ipcop/lang.pl';
require '/usr/lib/ipcop/header.pl';


# debug:
# 0 - no debug
# 1 - some output
my $debug = 0;

# path for rrd database
my $path_rrd = "/var/log/rrd";
# path for graphs
my $path_graphs = "/home/httpd/html/graphs";
# path for values
my $path_values = "/var/log/digitemp.log";
# settings
my %digitemp_settings = ();
if ( -e '/var/ipcop/addons/digitemp/settings' ) {
    &General::readhash('/var/ipcop/addons/digitemp/settings', \%digitemp_settings);
}

my %colors = ();
$colors{"1"} = "#0000FF";
$colors{"2"} = "#00FF00";
$colors{"3"} = "#FF0000";
$colors{"4"} = "#FFD700";
$colors{"5"} = "#CCCCCC";
$colors{"6"} = "#40E0D0";
$colors{"7"} = "#90EE90";
$colors{"8"} = "#F4A460";
$colors{"9"} = "#000000";     # someone with a nice RGB code for this ?
$colors{"10"} = "#808080";    # someone with a nice RGB code for this ?

my $sensors_max = 10;

my %values = ();

my $tty_port = $digitemp_settings{'COMPORT'} - 1;
my $digi_cmd = "/usr/local/bin/digitemp_$digitemp_settings{'ADAPTER'}";

my $i = 0;
my $j = 0;
my $display = 0;
my $line = '';
my $retry = 0;
my $ERROR;


##########
#
# commandline parameters
#
##########
while ( @ARGV ) {
    my $arg = shift @ARGV;

    system("/usr/bin/touch /var/ipcop/addons/digitemp/rescan") if ( ($arg eq 'scan') || ($arg eq 'rescan') );
    $debug = 1 if ( $arg eq 'debug' );
    $display = 1 if ( $arg eq 'display' );
}
$display = 1  if ( $debug >= 1 );

##########
#
# check for digitemp.conf file, scan for sensors if none exist
#
##########
if ( ! -e '/var/ipcop/addons/digitemp/digitemp.conf' || -e '/var/ipcop/addons/digitemp/rescan' ) {
    unlink('/var/ipcop/addons/digitemp/rescan');

    # no conf or manually forced, search for sensors
    print "Searching for sensors on /dev/ttyS$tty_port\n";
    `$digi_cmd -i -r 2500 -s /dev/ttyS$tty_port -c /var/ipcop/addons/digitemp/digitemp.conf`;

    # still no config file
    die "No sensors found" if ( ! -e '/var/ipcop/addons/digitemp/digitemp.conf' );

    # get rid of (old) graphs
    system("rm -f $path_graphs/digitemp-*");

    $digitemp_settings{'SENSORS'} = `cat /var/ipcop/addons/digitemp/digitemp.conf | grep SENSORS | /usr/bin/cut -d ' ' -f2`;
    chomp($digitemp_settings{'SENSORS'});

    print "$digitemp_settings{'SENSORS'} sensor(s) found\n";

    # remove sensor SerialNumbers from settings file
    for ($i = 0; $i < $sensors_max; $i++) {
        if ($digitemp_settings{"SENSOR$i"} ne '') {
            $digitemp_settings{"SENSOR$i"} = '';
        }
    }

    &General::writehash('/var/ipcop/addons/digitemp/settings', \%digitemp_settings);
}

# Added for conversion of utf-8 characters
#use Encode 'from_to';
#my %tr=();

# From makegraphs:
# Force language back to English (ugly hack!)
# Modified to only force if we are unable to convert charset 
# from utf-8
if (${Lang::language} =~ /^(ar|bg|el|fa|gu|ja|ru|th|ur|vi|zh|zt)$/) {
    &Lang::reload(2);
}

##########
#
# scan all sensors, retry 5 times in case of error
#
##########
for($i = 0; $i < $digitemp_settings{'SENSORS'}; $i++) {
    for (my $retries = 0; $retries < 5; $retries++) {
        $line = `$digi_cmd -q -t $i -o "Sensor %s SNR %R Celsius %.2C Hum %h" -H "Sensor %s SNR %R Celsius %.2C Hum %h" -c /var/ipcop/addons/digitemp/digitemp.conf`;
        chomp($line);
        print "$line\n" if ( $display > 0 );

        $line =~ /Sensor\s(\d+)\sSNR\s(.*)\sCelsius\s(.*)\sHum\s(.*)/;
        if ((index($line, "Sensor") != -1) && ($3 ne '85.00')) {
            $values{$1}{'SNR'} = $2;
            $values{$1}{'T'} = $3;
            $values{$1}{'H'} = $4;
# use this to get some dummy values for Humidity
#            $values{$1}{'H'} = 30 + 10 * $1;
            last;
        }
    }
}


##########
#
# write sensor data to logfile (for CGI), update RRDs, make graph
#
##########
open(FILE_VALUES, ">$path_values");
foreach my $key (sort keys %values) {
    if ($digitemp_settings{"SENSOR$key"} ne $values{$key}{'SNR'}) {
        $digitemp_settings{"SENSOR$key"} = $values{$key}{'SNR'};
        &General::writehash('/var/ipcop/addons/digitemp/settings', \%digitemp_settings);
        print "Sensor $key $values{$key}{'SNR'}\n";
    }
    print FILE_VALUES "$key,$values{$key}{'SNR'},$values{$key}{'T'},$values{$key}{'H'}\n";

    # create RRD
    if ( ! -e "$path_rrd/digitemp$values{$key}{'SNR'}.rrd") {
        RRDs::create ("$path_rrd/digitemp$values{$key}{'SNR'}.rrd", "--step=300",
                "DS:temp:GAUGE:600:-50:150",        # temperature expected between -50 &deg;C and 150 &deg;C
                "DS:hum:GAUGE:600:0:100",           # humidity expected between 0 % and 100 %
                "RRA:AVERAGE:0.5:1:576",
                "RRA:AVERAGE:0.5:6:672",
                "RRA:AVERAGE:0.5:24:732",
                "RRA:AVERAGE:0.5:144:1460");
        $ERROR = RRDs::error;
        print "Error in RRD::create for digitemp $values{$key}{'SNR'}: $ERROR\n" if $ERROR;
    }

    # update RRD
    RRDs::update ("$path_rrd/digitemp$values{$key}{'SNR'}.rrd", 
            "-t", "temp:hum", 
            "N:$values{$key}{'T'}:$values{$key}{'H'}");

    $ERROR = RRDs::error;
    print "Error in RRD::update for digitemp $values{$key}{'SNR'}: $ERROR\n" if $ERROR;

    if ($digitemp_settings{'GRAPHPERSENSOR'} eq 'on') {
        updategraph ("day", $values{$key}{'SNR'});
        updategraph ("week", $values{$key}{'SNR'});
        updategraph ("month", $values{$key}{'SNR'});
        updategraph ("year", $values{$key}{'SNR'});

        if ($values{$key}{'H'} != -1) {
            updatehumgraph ("day", $values{$key}{'SNR'});
            updatehumgraph ("week", $values{$key}{'SNR'});
            updatehumgraph ("month", $values{$key}{'SNR'});
            updatehumgraph ("year", $values{$key}{'SNR'});
        }
    }
}

if ($digitemp_settings{'GRAPHPERSENSOR'} ne 'on') {
    updatesinglegraph ("day");
    updatesinglegraph ("week");
    updatesinglegraph ("month");
    updatesinglegraph ("year");
    updatesinglehumgraph ("day");
    updatesinglehumgraph ("week");
    updatesinglehumgraph ("month");
    updatesinglehumgraph ("year");
}
close(FILE_VALUES);


sub updategraph 
{
    my $period = $_[0];
    my $SNR    = $_[1];
    my $label  = $digitemp_settings{"LABEL-$SNR"};

    $label = "sensor $SNR" if ( $label eq "" );

    my @rrdargs = ();

    push @rrdargs, @{&rrd_header("digitemp-$SNR", $period, "$Lang::tr{'digitemp temperature'} ($Lang::tr{$period})", -1, -1)};

    push(@rrdargs, "-v °Celsius");
    push(@rrdargs, "DEF:temp=$path_rrd/digitemp$SNR.rrd:temp:AVERAGE");
    push(@rrdargs, "LINE2:temp#0000FF:$label\\j");
    push(@rrdargs, "GPRINT:temp:MIN:$Lang::tr{'digitemp minimal'}\\: %5.2lf °C");
    push(@rrdargs, "GPRINT:temp:AVERAGE:$Lang::tr{'average'}\\: %5.2lf °C");
    push(@rrdargs, "GPRINT:temp:MAX:$Lang::tr{'maximal'}\\: %5.2lf °C");
    push(@rrdargs, "GPRINT:temp:LAST:$Lang::tr{'current'}\\: %5.2lf °C\\j");

    RRDs::graph ( @rrdargs );
    $ERROR = RRDs::error;
    print "Error in RRD::graph for sensor $SNR: $ERROR\n" if $ERROR;
}

sub updatehumgraph 
{
    my $period = $_[0];
    my $SNR    = $_[1];
    my $label  = $digitemp_settings{"LABEL-$SNR"};

    $label = "sensor $SNR" if ( $label eq "" );
  
    my @rrdargs = ();


    push @rrdargs, @{&rrd_header("digitemp-$SNR-hum", $period, "$Lang::tr{'digitemp humidity'} ($Lang::tr{$period})", 0, 100)};
    push(@rrdargs, "DEF:hum=$path_rrd/digitemp$SNR.rrd:hum:AVERAGE");
    push(@rrdargs, "LINE2:hum#0000FF:$label\\j");
    push(@rrdargs, "GPRINT:hum:MIN:$Lang::tr{'digitemp minimal'}\\: %3.0lf %%");
    push(@rrdargs, "GPRINT:hum:AVERAGE:$Lang::tr{'average'}\\: %3.0lf %%");
    push(@rrdargs, "GPRINT:hum:MAX:$Lang::tr{'maximal'}\\: %3.0lf %%");
    push(@rrdargs, "GPRINT:hum:LAST:$Lang::tr{'current'}\\: %3.0lf %%\\j");

    RRDs::graph ( @rrdargs );
    $ERROR = RRDs::error;
    print "Error in RRD::graph for hum sensor $SNR: $ERROR\n" if $ERROR;
}

sub updatesinglegraph 
{
    my $period = $_[0];

    my @rrdargs = ();

    push @rrdargs, @{&rrd_header("digitemp", $period, "$Lang::tr{'digitemp temperature'} ($Lang::tr{$period})", -1, -1)};
    push(@rrdargs, "--vertical-label", "°Celsius");

    for($i = 0; $i < $digitemp_settings{'SENSORS'}; $i++) {
        my $SNR    = $digitemp_settings{"SENSOR$i"};
        my $label  = $digitemp_settings{"LABEL-$SNR"};

        $label = "sensor $SNR" if ( $label eq "" );

        $j = $i + 1;

        push(@rrdargs, "DEF:$i=$path_rrd/digitemp$SNR.rrd:temp:AVERAGE");
        push(@rrdargs, "LINE2:$i$colors{$j}:$label\\j");
        push(@rrdargs, "GPRINT:$i:MIN:$Lang::tr{'digitemp minimal'}\\: %5.2lf °C");
        push(@rrdargs, "GPRINT:$i:AVERAGE:$Lang::tr{'average'}\\: %5.2lf °C");
        push(@rrdargs, "GPRINT:$i:MAX:$Lang::tr{'maximal'}\\: %5.2lf °C");
        push(@rrdargs, "GPRINT:$i:LAST:$Lang::tr{'current'}\\: %5.2lf °C\\j");
        push @rrdargs, @{&rrd_lastupdate()};
    }

    RRDs::graph ( @rrdargs );
    $ERROR = RRDs::error;
    print "Error in RRD::graph for single graph: $ERROR\n" if $ERROR;
}

sub updatesinglehumgraph 
{
    my $period = $_[0];

    my @rrdargs = ();

    push @rrdargs, @{&rrd_header("digitemp-hum", $period, "$Lang::tr{'digitemp humidity'} ($Lang::tr{$period})", 0, 100)};

    $i = 0;
    foreach my $key (sort keys %values) {
        next if ( $values{$key}{'H'} == -1 );

        my $SNR    = $values{$key}{'SNR'};
        my $label  = $digitemp_settings{"LABEL-$SNR"};

        $label = "sensor $SNR" if ( $label eq "" );

        $i = $key + 1;

        push(@rrdargs, "DEF:$key=$path_rrd/digitemp$SNR.rrd:hum:AVERAGE");
        push(@rrdargs, "LINE2:$key$colors{$i}:$label\\j");
        push(@rrdargs, "GPRINT:$key:MIN:$Lang::tr{'digitemp minimal'}\\: %3.0lf %%");
        push(@rrdargs, "GPRINT:$key:AVERAGE:$Lang::tr{'average'}\\: %3.0lf %%");
        push(@rrdargs, "GPRINT:$key:MAX:$Lang::tr{'maximal'}\\: %3.0lf %%");
        push(@rrdargs, "GPRINT:$key:LAST:$Lang::tr{'current'}\\: %3.0lf %%\\j");
        push @rrdargs, @{&rrd_lastupdate()};
    }

    # only if we have a hum. sensor
    if ($i != 0) {
        RRDs::graph ( @rrdargs );
        $ERROR = RRDs::error;
        print "Error in RRD::graph for single hum graph: $ERROR\n" if $ERROR;
    }
}


sub rrd_header {
    my $name   = shift;
    my $period = shift;
    my $title  = shift;
    my $lower  = shift;
    my $upper  = shift;
    my $result = [];

    push @$result, "$path_graphs/$name-$period.png";
    push @$result, ("--title", "$title");
    push @$result, ("--start", "-1$period");
    push @$result, ("--width", 600);
    push @$result, ("--height", 100);
    if ($lower != -1) {
        push @$result, ("--lower-limit", $lower);
        push @$result, ("--upper-limit", $upper);
    }
    push @$result, "--rigid";
    push @$result, "--alt-y-grid";
    push @$result, "--lazy";
    push @$result, ("--imgformat", "PNG");
    # gets ignored in 1.3 for now
    push @$result, "--interlaced";
    push @$result, "--pango-markup";
    push @$result, ("--font", "TITLE:0:sans mono bold oblique");
    push @$result, ("--color", "SHADEA$Header::boxcolour");
    push @$result, ("--color", "SHADEB$Header::boxcolour");
    push @$result, ("--color", "BACK$Header::boxcolour");

    return $result;
}


sub rrd_lastupdate {
    my $result  = [];

    push @$result, "COMMENT:<span size='smaller'> </span>\\r";
    push @$result, "COMMENT:<span size='smaller'>Last update\\: ". RRDescape(scalar localtime()) ."</span>\\r";

    return $result;
}


# This from munin-graph which is: Copyright (C) 2002-2004 Jimmy Olsen, Audun Ytterdal
# Munin has some pretty cool RRD graphing.
sub RRDescape {
    my $text = shift;
    return undef if not defined $text;
    $text =~ s/\\/\\\\/g;
    $text =~ s/:/\\:/g;
    return $text;
}
